# -*- coding: utf-8 -*-
# Copyright (c) 2008-2014 Erik Svensson <erik.public@gmail.com>
# Licensed under the MIT license.

import logging

LOGGER = logging.getLogger('transmission-rpc')
LOGGER.setLevel(logging.ERROR)


def mirror_dict(source):
    """
    Creates a dictionary with all values as keys and all keys as values.
    """
    source.update({value: key for key, value in source.items()})
    return source


DEFAULT_PROTOCOL = 'http'
DEFAULT_HOST = '127.0.0.1'
DEFAULT_PORT = 9091
DEFAULT_PATH= '/transmission/rpc'
DEFAULT_TIMEOUT = 30.0

TR_PRI_LOW = -1
TR_PRI_NORMAL = 0
TR_PRI_HIGH = 1

PRIORITY = mirror_dict({
    'low': TR_PRI_LOW, 'normal': TR_PRI_NORMAL, 'high': TR_PRI_HIGH
})

TR_RATIOLIMIT_GLOBAL = 0  # follow the global settings
TR_RATIOLIMIT_SINGLE = 1  # override the global settings, seeding until a certain ratio
TR_RATIOLIMIT_UNLIMITED = 2  # override the global settings, seeding regardless of ratio

RATIO_LIMIT = mirror_dict({
    'global': TR_RATIOLIMIT_GLOBAL, 'single': TR_RATIOLIMIT_SINGLE,
    'unlimited': TR_RATIOLIMIT_UNLIMITED
})

TR_IDLELIMIT_GLOBAL = 0  # follow the global settings
TR_IDLELIMIT_SINGLE = 1  # override the global settings, seeding until a certain idle time
TR_IDLELIMIT_UNLIMITED = 2  # override the global settings, seeding regardless of activity

IDLE_LIMIT = mirror_dict({
    'global': TR_RATIOLIMIT_GLOBAL, 'single': TR_RATIOLIMIT_SINGLE,
    'unlimited': TR_RATIOLIMIT_UNLIMITED
})

# A note on argument maps
# These maps are used to verify *-set methods. The information is structured in
# a tree.
# set +- <argument1> -                  [<type>,  <added version>, <removed version>, <previous argument name>, <next argument name>, <description>]
#  |  +- <argument2> -                  [<type>,  <added version>, <removed version>, <previous argument name>, <next argument name>, <description>]
#  |                                           ,
# get +- <argument1> -                  [<type>,  <added version>, <removed version>, <previous argument name>, <next argument name>, <description>]
#     +- <argument2> -                  [<type>,  <added version>, <removed version>, <previous argument name>, <next argument name>, <description>]

# Arguments for torrent methods
TORRENT_ARGS = {
    'get': {
        'activityDate': (
            'number', 1, None, None, None,
            'Last time of upload or download activity.'
        ),
        'addedDate': (
            'number', 1, None, None, None,
            'The date when this torrent was first added.'
        ),
        'announceResponse': (
            'string', 1, 7, None, None,
            'The announce message from the tracker.'
        ),
        'announceURL': ('string', 1, 7, None, None, 'Current announce URL.'),
        'bandwidthPriority': (
            'number', 5, None, None, None,
            'Bandwidth priority. Low (-1), Normal (0) or High (1).'
        ),
        'comment': ('string', 1, None, None, None, 'Torrent comment.'),
        'corruptEver': (
            'number', 1, None, None, None,
            'Number of bytes of corrupt data downloaded.'
        ),
        'creator': ('string', 1, None, None, None, 'Torrent creator.'),
        'dateCreated': (
            'number', 1, None, None, None, 'Torrent creation date.'
        ),
        'desiredAvailable': (
            'number', 1, None, None, None,
            'Number of bytes available and left to be downloaded.'
        ),
        'doneDate': (
            'number', 1, None, None, None,
            'The date when the torrent finished downloading.'
        ),
        'downloadDir': (
            'string', 4, None, None, None,
            'The directory path where the torrent is downloaded to.'
        ),
        'downloadedEver': (
            'number', 1, None, None, None,
            'Number of bytes of good data downloaded.'
        ),
        'downloaders': ('number', 4, 7, None, None, 'Number of downloaders.'),
        'downloadLimit': (
            'number', 1, None, None, None, 'Download limit in Kbps.'
        ),
        'downloadLimited': (
            'boolean', 5, None, None, None, 'Download limit is enabled'
        ),
        'downloadLimitMode': (
            'number', 1, 5, None, None,
            'Download limit mode. 0 means global, 1 means single, 2 unlimited.'
        ),
        'error': (
            'number', 1, None, None, None,
            'Kind of error. 0 means OK, 1 means tracker warning, 2 means tracker error, 3 means local error.'
        ),
        'errorString': ('number', 1, None, None, None, 'Error message.'),
        'eta': (
            'number', 1, None, None, None,
            'Estimated number of seconds left when downloading or seeding. -1 means not available and -2 means unknown.'
        ),
        'etaIdle': (
            'number', 15, None, None, None,
            'Estimated number of seconds left until the idle time limit is reached. -1 means not available and -2 means unknown.'
        ),
        'files': (
            'array', 1, None, None, None,
            'Array of file object containing key, bytesCompleted, length and name.'
        ),
        'fileStats': (
            'array', 5, None, None, None,
            'Away of file statistics containing bytesCompleted, wanted and priority.'
        ),
        'hashString': (
            'string', 1, None, None, None,
            'Hashstring unique for the torrent even between sessions.'
        ),
        'haveUnchecked': (
            'number', 1, None, None, None, 'Number of bytes of partial pieces.'
        ),
        'haveValid': (
            'number', 1, None, None, None,
            'Number of bytes of checksum verified data.'
        ),
        'honorsSessionLimits': (
            'boolean', 5, None, None, None,
            'True if session upload limits are honored'
        ),
        'id': ('number', 1, None, None, None, 'Session unique torrent id.'),
        'isFinished': (
            'boolean', 9, None, None, None,
            'True if the torrent is finished. Downloaded and seeded.'
        ),
        'isPrivate': (
            'boolean', 1, None, None, None, 'True if the torrent is private.'
        ),
        'isStalled': (
            'boolean', 14, None, None, None,
            'True if the torrent has stalled (been idle for a long time).'
        ),
        'lastAnnounceTime': (
            'number', 1, 7, None, None, 'The time of the last announcement.'
        ),
        'lastScrapeTime': (
            'number', 1, 7, None, None,
            'The time af the last successful scrape.'
        ),
        'leechers': ('number', 1, 7, None, None, 'Number of leechers.'),
        'leftUntilDone': (
            'number', 1, None, None, None,
            'Number of bytes left until the download is done.'
        ),
        'magnetLink': (
            'string', 7, None, None, None, 'The magnet link for this torrent.'
        ),
        'manualAnnounceTime': (
            'number', 1, None, None, None,
            'The time until you manually ask for more peers.'
        ),
        'maxConnectedPeers': (
            'number', 1, None, None, None, 'Maximum of connected peers.'
        ),
        'metadataPercentComplete': (
            'number', 7, None, None, None,
            'Download progress of metadata. 0.0 to 1.0.'
        ),
        'name': ('string', 1, None, None, None, 'Torrent name.'),
        'nextAnnounceTime': (
            'number', 1, 7, None, None, 'Next announce time.'
        ),
        'nextScrapeTime': ('number', 1, 7, None, None, 'Next scrape time.'),
        'peer-limit': (
            'number', 5, None, None, None, 'Maximum number of peers.'
        ),
        'peers': ('array', 2, None, None, None, 'Array of peer objects.'),
        'peersConnected': (
            'number', 1, None, None, None,
            'Number of peers we are connected to.'
        ),
        'peersFrom': (
            'object', 1, None, None, None,
            'Object containing download peers counts for different peer types.'
        ),
        'peersGettingFromUs': (
            'number', 1, None, None, None,
            'Number of peers we are sending data to.'
        ),
        'peersKnown': (
            'number', 1, 13, None, None,
            'Number of peers that the tracker knows.'
        ),
        'peersSendingToUs': (
            'number', 1, None, None, None, 'Number of peers sending to us'
        ),
        'percentDone': (
            'double', 5, None, None, None,
            'Download progress of selected files. 0.0 to 1.0.'
        ),
        'pieces': (
            'string', 5, None, None, None,
            'String with base64 encoded bitfield indicating finished pieces.'
        ),
        'pieceCount': ('number', 1, None, None, None, 'Number of pieces.'),
        'pieceSize': (
            'number', 1, None, None, None, 'Number of bytes in a piece.'
        ),
        'priorities': (
            'array', 1, None, None, None, 'Array of file priorities.'
        ),
        'queuePosition': (
            'number', 14, None, None, None, 'The queue position.'
        ),
        'rateDownload': (
            'number', 1, None, None, None, 'Download rate in bps.'
        ),
        'rateUpload': ('number', 1, None, None, None, 'Upload rate in bps.'),
        'recheckProgress': (
            'double', 1, None, None, None, 'Progress of recheck. 0.0 to 1.0.'
        ),
        'secondsDownloading': ('number', 15, None, None, None, ''),
        'secondsSeeding': ('number', 15, None, None, None, ''),
        'scrapeResponse': (
            'string', 1, 7, None, None, 'Scrape response message.'
        ),
        'scrapeURL': ('string', 1, 7, None, None, 'Current scrape URL'),
        'seeders': (
            'number', 1, 7, None, None,
            'Number of seeders reported by the tracker.'
        ),
        'seedIdleLimit': (
            'number', 10, None, None, None, 'Idle limit in minutes.'
        ),
        'seedIdleMode': (
            'number', 10, None, None, None,
            'Use global (0), torrent (1), or unlimited (2) limit.'
        ),
        'seedRatioLimit': ('double', 5, None, None, None, 'Seed ratio limit.'),
        'seedRatioMode': (
            'number', 5, None, None, None,
            'Use global (0), torrent (1), or unlimited (2) limit.'
        ),
        'sizeWhenDone': (
            'number', 1, None, None, None,
            'Size of the torrent download in bytes.'
        ),
        'startDate': (
            'number', 1, None, None, None,
            'The date when the torrent was last started.'
        ),
        'status': (
            'number', 1, None, None, None, 'Current status, see source'
        ),
        'swarmSpeed': (
            'number', 1, 7, None, None, 'Estimated speed in Kbps in the swarm.'
        ),
        'timesCompleted': (
            'number', 1, 7, None, None,
            'Number of successful downloads reported by the tracker.'
        ),
        'trackers': (
            'array', 1, None, None, None, 'Array of tracker objects.'
        ),
        'trackerStats': (
            'object', 7, None, None, None,
            'Array of object containing tracker statistics.'
        ),
        'totalSize': (
            'number', 1, None, None, None, 'Total size of the torrent in bytes'
        ),
        'torrentFile': (
            'string', 5, None, None, None, 'Path to .torrent file.'
        ),
        'uploadedEver': (
            'number', 1, None, None, None, 'Number of bytes uploaded, ever.'
        ),
        'uploadLimit': ('number', 1, None, None, None, 'Upload limit in Kbps'),
        'uploadLimitMode': (
            'number', 1, 5, None, None,
            'Upload limit mode. 0 means global, 1 means single, 2 unlimited.'
        ),
        'uploadLimited': (
            'boolean', 5, None, None, None, 'Upload limit enabled.'
        ),
        'uploadRatio': ('double', 1, None, None, None, 'Seed ratio.'),
        'wanted': (
            'array', 1, None, None, None,
            'Array of booleans indicated wanted files.'
        ),
        'webseeds': (
            'array', 1, None, None, None, 'Array of webseeds objects'
        ),
        'webseedsSendingToUs': (
            'number', 1, None, None, None, 'Number of webseeds seeding to us.'
        ),
    }, 'set': {
        'bandwidthPriority': (
            'number', 5, None, None, None, 'Priority for this transfer.'
        ),
        'downloadLimit': (
            'number', 5, None, 'speed-limit-down', None,
            'Set the speed limit for download in Kib/s.'
        ),
        'downloadLimited': (
            'boolean', 5, None, 'speed-limit-down-enabled', None,
            'Enable download speed limiter.'
        ),
        'files-wanted': (
            'array', 1, None, None, None,
            "A list of file id's that should be downloaded."
        ),
        'files-unwanted': (
            'array', 1, None, None, None,
            "A list of file id's that shouldn't be downloaded."
        ),
        'honorsSessionLimits': (
            'boolean', 5, None, None, None,
            "Enables or disables the transfer to honour the upload limit set in the session."
        ),
        'location': ('array', 1, None, None, None, 'Local download location.'),
        'peer-limit': (
            'number', 1, None, None, None, 'The peer limit for the torrents.'
        ),
        'priority-high': (
            'array', 1, None, None, None,
            "A list of file id's that should have high priority."
        ),
        'priority-low': (
            'array', 1, None, None, None,
            "A list of file id's that should have normal priority."
        ),
        'priority-normal': (
            'array', 1, None, None, None,
            "A list of file id's that should have low priority."
        ),
        'queuePosition': (
            'number', 14, None, None, None,
            'Position of this transfer in its queue.'
        ),
        'seedIdleLimit': (
            'number', 10, None, None, None, 'Seed inactivity limit in minutes.'
        ),
        'seedIdleMode': (
            'number', 10, None, None, None,
            'Seed inactivity mode. 0 = Use session limit, 1 = Use transfer limit, 2 = Disable limit.'
        ),
        'seedRatioLimit': ('double', 5, None, None, None, 'Seeding ratio.'),
        'seedRatioMode': (
            'number', 5, None, None, None,
            'Which ratio to use. 0 = Use session limit, 1 = Use transfer limit, 2 = Disable limit.'
        ),
        'speed-limit-down': (
            'number', 1, 5, None, 'downloadLimit',
            'Set the speed limit for download in Kib/s.'
        ),
        'speed-limit-down-enabled': (
            'boolean', 1, 5, None, 'downloadLimited',
            'Enable download speed limiter.'
        ),
        'speed-limit-up': (
            'number', 1, 5, None, 'uploadLimit',
            'Set the speed limit for upload in Kib/s.'
        ),
        'speed-limit-up-enabled': (
            'boolean', 1, 5, None, 'uploadLimited',
            'Enable upload speed limiter.'
        ),
        'trackerAdd': (
            'array', 10, None, None, None,
            'Array of string with announce URLs to add.'
        ),
        'trackerRemove': (
            'array', 10, None, None, None,
            'Array of ids of trackers to remove.'
        ),
        'trackerReplace': (
            'array', 10, None, None, None,
            'Array of (id, url) tuples where the announce URL should be replaced.'
        ),
        'uploadLimit': (
            'number', 5, None, 'speed-limit-up', None,
            'Set the speed limit for upload in Kib/s.'
        ),
        'uploadLimited': (
            'boolean', 5, None, 'speed-limit-up-enabled', None,
            'Enable upload speed limiter.'
        ),
    }, 'add': {
        'bandwidthPriority': (
            'number', 8, None, None, None, 'Priority for this transfer.'
        ),
        'download-dir': (
            'string', 1, None, None, None,
            'The directory where the downloaded contents will be saved in.'
        ),
        'cookies': (
            'string', 13, None, None, None, 'One or more HTTP cookie(s).'
        ),
        'filename': (
            'string', 1, None, None, None,
            "A file path or URL to a torrent file or a magnet link."
        ),
        'files-wanted': (
            'array', 1, None, None, None,
            "A list of file id's that should be downloaded."
        ),
        'files-unwanted': (
            'array', 1, None, None, None,
            "A list of file id's that shouldn't be downloaded."
        ),
        'metainfo': (
            'string', 1, None, None, None,
            'The content of a torrent file, base64 encoded.'
        ),
        'paused': (
            'boolean', 1, None, None, None,
            'If True, does not start the transfer when added.'
        ),
        'peer-limit': (
            'number', 1, None, None, None, 'Maximum number of peers allowed.'
        ),
        'priority-high': (
            'array', 1, None, None, None,
            "A list of file id's that should have high priority."
        ),
        'priority-low': (
            'array', 1, None, None, None,
            "A list of file id's that should have low priority."
        ),
        'priority-normal': (
            'array', 1, None, None, None,
            "A list of file id's that should have normal priority."
        ),
    }
}

# Arguments for session methods
SESSION_ARGS = {
    'get': {
        "alt-speed-down": (
            'number', 5, None, None, None,
            'Alternate session download speed limit (in Kib/s).'
        ),
        "alt-speed-enabled": (
            'boolean', 5, None, None, None,
            'True if alternate global download speed limiter is enabled.'
        ),
        "alt-speed-time-begin": (
            'number', 5, None, None, None,
            'Time when alternate speeds should be enabled. Minutes after midnight.'
        ),
        "alt-speed-time-enabled": (
            'boolean', 5, None, None, None,
            'True if alternate speeds scheduling is enabled.'
        ),
        "alt-speed-time-end": (
            'number', 5, None, None, None,
            'Time when alternate speeds should be disabled. Minutes after midnight.'
        ),
        "alt-speed-time-day": (
            'number', 5, None, None, None,
            'Days alternate speeds scheduling is enabled.'
        ),
        "alt-speed-up": (
            'number', 5, None, None, None,
            'Alternate session upload speed limit (in Kib/s)'
        ),
        "blocklist-enabled": (
            'boolean', 5, None, None, None, 'True when blocklist is enabled.'
        ),
        "blocklist-size": (
            'number', 5, None, None, None, 'Number of rules in the blocklist'
        ),
        "blocklist-url": (
            'string', 11, None, None, None,
            'Location of the block list. Updated with blocklist-update.'
        ),
        "cache-size-mb": (
            'number', 10, None, None, None,
            'The maximum size of the disk cache in MB'
        ),
        "config-dir": (
            'string', 8, None, None, None,
            'location of transmissions configuration directory'
        ),
        "dht-enabled": (
            'boolean', 6, None, None, None, 'True if DHT enabled.'
        ),
        "download-dir": (
            'string', 1, None, None, None, 'The download directory.'
        ),
        "download-dir-free-space": (
            'number', 12, None, None, None,
            'Free space in the download directory, in bytes'
        ),
        "download-queue-size": (
            'number', 14, None, None, None,
            'Number of slots in the download queue.'
        ),
        "download-queue-enabled": (
            'boolean', 14, None, None, None,
            'True if the download queue is enabled.'
        ),
        "encryption": (
            'string', 1, None, None, None,
            'Encryption mode, one of ``required``, ``preferred`` or ``tolerated``.'
        ),
        "idle-seeding-limit": (
            'number', 10, None, None, None, 'Seed inactivity limit in minutes.'
        ),
        "idle-seeding-limit-enabled": (
            'boolean', 10, None, None, None,
            'True if the seed activity limit is enabled.'
        ),
        "incomplete-dir": (
            'string', 7, None, None, None,
            'The path to the directory for incomplete torrent transfer data.'
        ),
        "incomplete-dir-enabled": (
            'boolean', 7, None, None, None,
            'True if the incomplete dir is enabled.'
        ),
        "lpd-enabled": (
            'boolean', 9, None, None, None,
            'True if local peer discovery is enabled.'
        ),
        "peer-limit": (
            'number', 1, 5, None, 'peer-limit-global',
            'Maximum number of peers.'
        ),
        "peer-limit-global": (
            'number', 5, None, 'peer-limit', None, 'Maximum number of peers.'
        ),
        "peer-limit-per-torrent": (
            'number', 5, None, None, None,
            'Maximum number of peers per transfer.'
        ),
        "pex-allowed": (
            'boolean', 1, 5, None, 'pex-enabled', 'True if PEX is allowed.'
        ),
        "pex-enabled": (
            'boolean', 5, None, 'pex-allowed', None, 'True if PEX is enabled.'
        ),
        "port": ('number', 1, 5, None, 'peer-port', 'Peer port.'),
        "peer-port": ('number', 5, None, 'port', None, 'Peer port.'),
        "peer-port-random-on-start": (
            'boolean', 5, None, None, None,
            'Enables randomized peer port on start of Transmission.'
        ),
        "port-forwarding-enabled": (
            'boolean', 1, None, None, None,
            'True if port forwarding is enabled.'
        ),
        "queue-stalled-minutes": (
            'number', 14, None, None, None,
            'Number of minutes of idle that marks a transfer as stalled.'
        ),
        "queue-stalled-enabled": (
            'boolean', 14, None, None, None,
            'True if stalled tracking of transfers is enabled.'
        ),
        "rename-partial-files": (
            'boolean', 8, None, None, None,
            'True if ".part" is appended to incomplete files'
        ),
        "rpc-version": (
            'number', 4, None, None, None, 'Transmission RPC API Version.'
        ),
        "rpc-version-minimum": (
            'number', 4, None, None, None, 'Minimum accepted RPC API Version.'
        ),
        "script-torrent-done-enabled": (
            'boolean', 9, None, None, None,
            'True if the done script is enabled.'
        ),
        "script-torrent-done-filename": (
            'string', 9, None, None, None,
            'Filename of the script to run when the transfer is done.'
        ),
        "seedRatioLimit": (
            'double', 5, None, None, None,
            'Seed ratio limit. 1.0 means 1:1 download and upload ratio.'
        ),
        "seedRatioLimited": (
            'boolean', 5, None, None, None,
            'True if seed ration limit is enabled.'
        ),
        "seed-queue-size": (
            'number', 14, None, None, None,
            'Number of slots in the upload queue.'
        ),
        "seed-queue-enabled": (
            'boolean', 14, None, None, None, 'True if upload queue is enabled.'
        ),
        "speed-limit-down": (
            'number', 1, None, None, None, 'Download speed limit (in Kib/s).'
        ),
        "speed-limit-down-enabled": (
            'boolean', 1, None, None, None,
            'True if the download speed is limited.'
        ),
        "speed-limit-up": (
            'number', 1, None, None, None, 'Upload speed limit (in Kib/s).'
        ),
        "speed-limit-up-enabled": (
            'boolean', 1, None, None, None,
            'True if the upload speed is limited.'
        ),
        "start-added-torrents": (
            'boolean', 9, None, None, None,
            'When true uploaded torrents will start right away.'
        ),
        "trash-original-torrent-files": (
            'boolean', 9, None, None, None,
            'When true added .torrent files will be deleted.'
        ),
        'units': (
            'object', 10, None, None, None,
            'An object containing units for size and speed.'
        ),
        'utp-enabled': (
            'boolean', 13, None, None, None,
            'True if Micro Transport Protocol (UTP) is enabled.'
        ),
        "version": ('string', 3, None, None, None, 'Transmission version.'),
    },
    'set': {
        "alt-speed-down": (
            'number', 5, None, None, None,
            'Alternate session download speed limit (in Kib/s).'
        ),
        "alt-speed-enabled": (
            'boolean', 5, None, None, None,
            'Enables alternate global download speed limiter.'
        ),
        "alt-speed-time-begin": (
            'number', 5, None, None, None,
            'Time when alternate speeds should be enabled. Minutes after midnight.'
        ),
        "alt-speed-time-enabled": (
            'boolean', 5, None, None, None,
            'Enables alternate speeds scheduling.'
        ),
        "alt-speed-time-end": (
            'number', 5, None, None, None,
            'Time when alternate speeds should be disabled. Minutes after midnight.'
        ),
        "alt-speed-time-day": (
            'number', 5, None, None, None,
            'Enables alternate speeds scheduling these days.'
        ),
        "alt-speed-up": (
            'number', 5, None, None, None,
            'Alternate session upload speed limit (in Kib/s).'
        ),
        "blocklist-enabled": (
            'boolean', 5, None, None, None, 'Enables the block list'
        ),
        "blocklist-url": (
            'string', 11, None, None, None,
            'Location of the block list. Updated with blocklist-update.'
        ),
        "cache-size-mb": (
            'number', 10, None, None, None,
            'The maximum size of the disk cache in MB'
        ),
        "dht-enabled": ('boolean', 6, None, None, None, 'Enables DHT.'),
        "download-dir": (
            'string', 1, None, None, None,
            'Set the session download directory.'
        ),
        "download-queue-size": (
            'number', 14, None, None, None,
            'Number of slots in the download queue.'
        ),
        "download-queue-enabled": (
            'boolean', 14, None, None, None, 'Enables download queue.'
        ),
        "encryption": (
            'string', 1, None, None, None,
            'Set the session encryption mode, one of ``required``, ``preferred`` or ``tolerated``.'
        ),
        "idle-seeding-limit": (
            'number', 10, None, None, None,
            'The default seed inactivity limit in minutes.'
        ),
        "idle-seeding-limit-enabled": (
            'boolean', 10, None, None, None,
            'Enables the default seed inactivity limit'
        ),
        "incomplete-dir": (
            'string', 7, None, None, None,
            'The path to the directory of incomplete transfer data.'
        ),
        "incomplete-dir-enabled": (
            'boolean', 7, None, None, None,
            'Enables the incomplete transfer data directory. Otherwise data for incomplete transfers are stored in the download target.'
        ),
        "lpd-enabled": (
            'boolean', 9, None, None, None,
            'Enables local peer discovery for public torrents.'
        ),
        "peer-limit": (
            'number', 1, 5, None, 'peer-limit-global',
            'Maximum number of peers.'
        ),
        "peer-limit-global": (
            'number', 5, None, 'peer-limit', None, 'Maximum number of peers.'
        ),
        "peer-limit-per-torrent": (
            'number', 5, None, None, None,
            'Maximum number of peers per transfer.'
        ),
        "pex-allowed": (
            'boolean', 1, 5, None, 'pex-enabled',
            'Allowing PEX in public torrents.'
        ),
        "pex-enabled": (
            'boolean', 5, None, 'pex-allowed', None,
            'Allowing PEX in public torrents.'
        ),
        "port": ('number', 1, 5, None, 'peer-port', 'Peer port.'),
        "peer-port": ('number', 5, None, 'port', None, 'Peer port.'),
        "peer-port-random-on-start": (
            'boolean', 5, None, None, None,
            'Enables randomized peer port on start of Transmission.'
        ),
        "port-forwarding-enabled": (
            'boolean', 1, None, None, None, 'Enables port forwarding.'
        ),
        "rename-partial-files": (
            'boolean', 8, None, None, None,
            'Appends ".part" to incomplete files'
        ),
        "queue-stalled-minutes": (
            'number', 14, None, None, None,
            'Number of minutes of idle that marks a transfer as stalled.'
        ),
        "queue-stalled-enabled": (
            'boolean', 14, None, None, None,
            'Enable tracking of stalled transfers.'
        ),
        "script-torrent-done-enabled": (
            'boolean', 9, None, None, None,
            'Whether or not to call the "done" script.'
        ),
        "script-torrent-done-filename": (
            'string', 9, None, None, None,
            'Filename of the script to run when the transfer is done.'
        ),
        "seed-queue-size": (
            'number', 14, None, None, None,
            'Number of slots in the upload queue.'
        ),
        "seed-queue-enabled": (
            'boolean', 14, None, None, None, 'Enables upload queue.'
        ),
        "seedRatioLimit": (
            'double', 5, None, None, None,
            'Seed ratio limit. 1.0 means 1:1 download and upload ratio.'
        ),
        "seedRatioLimited": (
            'boolean', 5, None, None, None, 'Enables seed ration limit.'
        ),
        "speed-limit-down": (
            'number', 1, None, None, None, 'Download speed limit (in Kib/s).'
        ),
        "speed-limit-down-enabled": (
            'boolean', 1, None, None, None, 'Enables download speed limiting.'
        ),
        "speed-limit-up": (
            'number', 1, None, None, None, 'Upload speed limit (in Kib/s).'
        ),
        "speed-limit-up-enabled": (
            'boolean', 1, None, None, None, 'Enables upload speed limiting.'
        ),
        "start-added-torrents": (
            'boolean', 9, None, None, None,
            'Added torrents will be started right away.'
        ),
        "trash-original-torrent-files": (
            'boolean', 9, None, None, None,
            'The .torrent file of added torrents will be deleted.'
        ),
        'utp-enabled': (
            'boolean', 13, None, None, None,
            'Enables Micro Transport Protocol (UTP).'
        ),
    },
}



# -*- coding: utf-8 -*-
# 2008-12, Erik Svensson <erik.public@gmail.com>
# Licensed under the MIT license.

import datetime
import unittest
import transmission_rpc.utils as tu



class utils(unittest.TestCase):
    def testFormatSize(self):
        table = {
            512                : (512, 'B'),
            1024               : (1.0, 'KiB'),
            1048575            : (1023.999, 'KiB'),
            1048576            : (1.0, 'MiB'),
            1073741824         : (1.0, 'GiB'),
            1099511627776      : (1.0, 'TiB'),
            1125899906842624   : (1.0, 'PiB'),
            1152921504606846976: (1.0, 'EiB'),
        }
        for size, expected in table.items():
            result = tu.format_size(size)
            self.assertAlmostEqual(result[0], expected[0], 4)
            self.assertEqual(result[1], expected[1])

    def testFormatSpeed(self):
        table = {
            512                : (512, 'B/s'),
            1024               : (1.0, 'KiB/s'),
            1048575            : (1023.999, 'KiB/s'),
            1048576            : (1.0, 'MiB/s'),
            1073741824         : (1.0, 'GiB/s'),
            1099511627776      : (1.0, 'TiB/s'),
            1125899906842624   : (1.0, 'PiB/s'),
            1152921504606846976: (1.0, 'EiB/s'),
        }
        for size, expected in table.items():
            result = tu.format_speed(size)
            self.assertAlmostEqual(result[0], expected[0], 4)
            self.assertEqual(result[1], expected[1])

    def testFormatTimedelta(self):
        table = {
            datetime.timedelta(0, 0)     : '0 00:00:00',
            datetime.timedelta(0, 10)    : '0 00:00:10',
            datetime.timedelta(0, 60)    : '0 00:01:00',
            datetime.timedelta(0, 61)    : '0 00:01:01',
            datetime.timedelta(0, 3661)  : '0 01:01:01',
            datetime.timedelta(1, 3661)  : '1 01:01:01',
            datetime.timedelta(13, 65660): '13 18:14:20',
        }
        for delta, expected in table.items():
            self.assertEqual(tu.format_timedelta(delta), expected)

    def testFormatTimestamp(self):
        table = {
            0         : '-',
            1         : '1970-01-01 00:00:01',
            1129135532: '2005-10-12 16:45:32',
        }
        for timestamp, expected in table.items():
            self.assertEqual(tu.format_timestamp(timestamp, utc=True), expected)

    def testInetAddress(self):
        table = {
            ('127.0.0.1:80', 2000)    : ('127.0.0.1', 80),
            ('127.0.0.1', 2000)       : ('127.0.0.1', 2000),
            (':80', 2000)             : ('localhost', 80),
            (':80', 2000, '127.0.0.1'): ('127.0.0.1', 80),
            ('0.0.0.0:443', 2000)     : ('0.0.0.0', 443),
            ('localhost:443', 2000)   : ('localhost', 443),
        }
        for args, expected in table.items():
            self.assertEqual(tu.inet_address(*args), expected)

        self.assertRaises(tu.INetAddressError, tu.inet_address, '256.256.256.256', 2000)

    def testRPCBool(self):
        table = {
            0      : 0,
            1      : 1,
            1000   : 1,
            'true' : 1,
            'Yes'  : 1,
            'truE' : 1,
            'baka' : 0,
            'false': 0,
            'no'   : 0,
            True   : 1,
            False  : 0,
        }
        for value, expected in table.items():
            self.assertEqual(tu.rpc_bool(value), expected)


def suite():
    suite = unittest.TestLoader().loadTestsFromTestCase(utils)
    return suite


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2014 Erik Svensson <erik.public@gmail.com>
# Licensed under the MIT license.

import socket
import datetime
import logging
from collections import namedtuple
import transmission_rpc.constants as constants
from transmission_rpc.constants import LOGGER

UNITS = ['B', 'KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB']


def format_size(size):
    """
    Format byte size into IEC prefixes, B, KiB, MiB ...
    """
    size = float(size)
    i = 0
    while size >= 1024.0 and i < len(UNITS):
        i += 1
        size /= 1024.0
    return size, UNITS[i]


def format_speed(size):
    """
    Format bytes per second speed into IEC prefixes, B/s, KiB/s, MiB/s ...
    """
    (size, unit) = format_size(size)
    return size, unit + '/s'


def format_timedelta(delta):
    """
    Format datetime.timedelta into <days> <hours>:<minutes>:<seconds>.
    """
    minutes, seconds = divmod(delta.seconds, 60)
    hours, minutes = divmod(minutes, 60)
    return '%d %02d:%02d:%02d' % (delta.days, hours, minutes, seconds)


def format_timestamp(timestamp, utc=False):
    """
    Format unix timestamp into ISO date format.
    """
    if timestamp > 0:
        if utc:
            dt_timestamp = datetime.datetime.utcfromtimestamp(timestamp)
        else:
            dt_timestamp = datetime.datetime.fromtimestamp(timestamp)
        return dt_timestamp.isoformat(' ')
    else:
        return '-'


class INetAddressError(Exception):
    """
    Error parsing / generating a internet address.
    """
    pass


def inet_address(address, default_port, default_address='localhost'):
    """
    Parse internet address.
    """
    addr = address.split(':')
    if len(addr) == 1:
        try:
            port = int(addr[0])
            addr = default_address
        except ValueError:
            addr = addr[0]
            port = default_port
    elif len(addr) == 2:
        try:
            port = int(addr[1])
        except ValueError:
            raise INetAddressError('Invalid address "%s".' % address)
        if len(addr[0]) == 0:
            addr = default_address
        else:
            addr = addr[0]
    else:
        raise INetAddressError('Invalid address "%s".' % address)
    try:
        socket.getaddrinfo(addr, port, socket.AF_INET, socket.SOCK_STREAM)
    except socket.gaierror:
        raise INetAddressError('Cannot look up address "%s".' % address)
    return addr, port


def rpc_bool(arg):
    """
    Convert between Python boolean and Transmission RPC boolean.
    """
    if isinstance(arg, str):
        try:
            arg = bool(int(arg))
        except ValueError:
            arg = arg.lower() in ['true', 'yes']
    return 1 if bool(arg) else 0


TR_TYPE_MAP = {
    'number' : int,
    'string' : str,
    'double' : float,
    'boolean': rpc_bool,
    'array'  : list,
    'object' : dict
}


def make_python_name(name):
    """
    Convert Transmission RPC name to python compatible name.
    """
    return name.replace('-', '_')


def make_rpc_name(name):
    """
    Convert python compatible name to Transmission RPC name.
    """
    return name.replace('_', '-')


def argument_value_convert(method, argument, value, rpc_version):
    """
    Check and fix Transmission RPC issues with regards to methods, arguments and values.
    """
    if method in ('torrent-add', 'torrent-get', 'torrent-set'):
        args = constants.TORRENT_ARGS[method[-3:]]
    elif method in ('session-get', 'session-set'):
        args = constants.SESSION_ARGS[method[-3:]]
    else:
        return ValueError('Method "%s" not supported' % (method))
    if argument in args:
        info = args[argument]
        invalid_version = True
        while invalid_version:
            invalid_version = False
            replacement = None
            if rpc_version < info[1]:
                invalid_version = True
                replacement = info[3]
            if info[2] and info[2] <= rpc_version:
                invalid_version = True
                replacement = info[4]
            if invalid_version:
                if replacement:
                    LOGGER.warning(
                        'Replacing requested argument "%s" with "%s".'
                        % (argument, replacement))
                    argument = replacement
                    info = args[argument]
                else:
                    raise ValueError(
                        'Method "%s" Argument "%s" does not exist in version %d.'
                        % (method, argument, rpc_version))
        return (argument, TR_TYPE_MAP[info[0]](value))
    else:
        raise ValueError('Argument "%s" does not exists for method "%s".',
                         (argument, method))


def get_arguments(method, rpc_version):
    """
    Get arguments for method in specified Transmission RPC version.
    """
    if method in ('torrent-add', 'torrent-get', 'torrent-set'):
        args = constants.TORRENT_ARGS[method[-3:]]
    elif method in ('session-get', 'session-set'):
        args = constants.SESSION_ARGS[method[-3:]]
    else:
        return ValueError('Method "%s" not supported' % (method))
    accessible = []
    for argument, info in args.items():
        valid_version = True
        if rpc_version < info[1]:
            valid_version = False
        if info[2] and info[2] <= rpc_version:
            valid_version = False
        if valid_version:
            accessible.append(argument)
    return accessible


Field = namedtuple('Field', ['value', 'dirty'])



# -*- coding: utf-8 -*-
# 2008-12, Erik Svensson <erik.public@gmail.com>
# Licensed under the MIT license.

import unittest
from . import top, client, torrent, utils


def test():
    tests = unittest.TestSuite([top.suite(), utils.suite(), torrent.suite(), client.suite()])
    result = unittest.TestResult()
    tests.run(result, True)
    for e in result.errors:
        for m in e:
            print(m)


test()



# -*- coding: utf-8 -*-
# 2013-03, Erik Svensson <erik.public@gmail.com>
# Licensed under the MIT license.

import unittest
import transmission_rpc


class TopTest(unittest.TestCase):

    def testConstants(self):
        self.assertTrue(isinstance(transmission_rpc.__author__, str))
        self.assertTrue(isinstance(transmission_rpc.__version__, str))
        self.assertTrue(isinstance(transmission_rpc.__author_email__, str))
        self.assertTrue(isinstance(transmission_rpc.__copyright__, str))
        self.assertTrue(isinstance(transmission_rpc.__license__, str))


def suite():
    suite = unittest.TestLoader().loadTestsFromTestCase(TopTest)
    return suite



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2014 Erik Svensson <erik.public@gmail.com>
# Licensed under the MIT license.


class TransmissionError(Exception):
    """
        This exception is raised when there has occurred an error related to
        communication with Transmission. It is a subclass of Exception.
    """
    def __init__(self, message='', original=None):
        Exception.__init__(self)
        self.message = message
        self.original = original

    def __str__(self):
        if self.original:
            original_name = type(self.original).__name__
            return '%s Original exception: %s, "%s"' % (
                self.message, original_name, str(self.original)
            )
        else:
            return self.message


class TransmissionAuthError(TransmissionError):
    pass



# -*- coding: utf-8 -*-
# 2008-12, Erik Svensson <erik.public@gmail.com>
# Licensed under the MIT license.

import os
import unittest
import base64
import json
import transmission_rpc.constants
from transmission_rpc import Client

from urllib.parse import urlparse

TEST_HOST = os.getenv('TRANSMISSION_HOST', '127.0.0.1')


def tree_differences(a, b):
    return node_differences(a, b, '.')


def node_differences(a, b, root):
    errors = []
    if isinstance(a, dict) and isinstance(b, dict):
        for k, v in a.items():
            node = root + '.' + k
            if k not in b:
                errors.append('Field %s missing from b at %s' % (k, node))
            else:
                errors.extend(node_differences(a[k], b[k], node))
        for k, v in b.items():
            node = root + '.' + k
            if k not in a:
                errors.append('Field %s missing from a at %s' % (k, node))
    elif isinstance(a, list) and isinstance(b, list):
        for v in a:
            if v not in b:
                errors.append(
                    'Value %s missing from b at %s' % (v[0:32], root)
                )
        for v in b:
            if v not in a:
                errors.append(
                    'Value %s missing from a at %s' % (v[0:32], root)
                )
    else:
        if a != b:
            errors.append('Value %s != %s at %s' % (a[0:32], b[0:32], root))
    return errors


class TestHTTPHandler(HTTPHandler):
    def __init__(self, test_name=None):
        self.url = None
        self.user = None
        self.password = None
        self.tests = None
        self.test_index = 0
        if test_name:
            test_file = test_name + '.json'
            here = os.path.dirname(os.path.abspath(__file__))
            test_path = os.path.join(here, 'data', test_file)
            fd = open(test_path, 'r')
            test_data = json.load(fd)
            fd.close()
            if 'test sequence' in test_data:
                self.tests = test_data['test sequence']

    def set_authentication(self, url, user, password):
        url_object = urlparse(url)
        if url_object.scheme == '':
            raise ValueError('URL should have a scheme.')
        else:
            self.url = url
        if user and password:
            if isinstance(user, str):
                self.user = user
            else:
                raise TypeError('Invalid type for user.')
            if isinstance(password, str):
                self.password = password
            else:
                raise TypeError('Invalid type for password.')
        elif user or password:
            raise ValueError('User AND password or neither.')

    def request(self, url, query, headers, timeout):
        response = {}
        if self.url and self.url != url:
            raise ValueError('New URL?!')
        url_obj = urlparse(url)
        if url_obj.scheme == '':
            raise ValueError('URL should have a scheme.')
        else:
            self.url = url
        q = json.loads(query)

        if self.tests:
            test_data = self.tests[self.test_index]
            self.test_index += 1
            errors = tree_differences(test_data['request'], q)
            if len(errors) > 0:
                errors = '\n\t'.join(errors)
                raise Exception(
                    'Invalid request\n%s\n%s\n. Errors: %s\n' % (
                        json.dumps(q, indent=2),
                        json.dumps(test_data['request'], indent=2), errors
                    )
                )
            if 'response' in test_data:
                response = test_data['response']
        else:
            response['tag'] = int(q['tag'])
            response['result'] = 'success'
        return json.dumps(response)


def createClient(*args, **kwargs):
    test_name = None
    if 'test_name' in kwargs:
        test_name = kwargs['test_name']
        del kwargs['test_name']
    kwargs['http_handler'] = TestHTTPHandler(test_name)
    return Client(*args, **kwargs)


class ClientTest(unittest.TestCase):
    def testConstruction(self):
        tc = createClient(test_name='construction')
        self.assertEqual(
            tc.url, 'http://localhost:{}/transmission/rpc'.format(
                transmission_rpc.constants.DEFAULT_PORT
            )
        )
        tc = createClient(
            '127.0.0.1',
            7000,
            user='user',
            password='secret',
            test_name='construction'
        )
        self.assertEqual(tc.url, 'http://127.0.0.1:7000/transmission/rpc')
        tc = createClient(
            '127.0.0.1',
            7000,
            user='user',
            password='secret',
            test_name='construction'
        )
        self.assertEqual(tc.url, 'http://127.0.0.1:7000/transmission/rpc')
        tc = createClient(
            '127.0.0.1', 7000, user='user', test_name='construction'
        )
        self.assertEqual(tc.url, 'http://127.0.0.1:7000/transmission/rpc')
        tc = createClient(
            '127.0.0.1', 7000, password='secret', test_name='construction'
        )
        self.assertEqual(tc.url, 'http://127.0.0.1:7000/transmission/rpc')
        tc = createClient(
            '127.0.0.1',
            7000,
            password='secret',
            timeout=0.1,
            test_name='construction'
        )
        self.assertEqual(tc.url, 'http://127.0.0.1:7000/transmission/rpc')
        self.assertEqual(tc.timeout, 0.1)
        tc = createClient(
            '127.0.0.1',
            7000,
            password='secret',
            timeout=10,
            test_name='construction'
        )
        self.assertEqual(tc.timeout, 10.0)
        tc = createClient(
            '127.0.0.1',
            7000,
            password='secret',
            timeout=10,
            test_name='construction'
        )
        self.assertEqual(tc.timeout, 10.0)

    def testTimeoutProperty(self):
        tc = createClient(
            '127.0.0.1', 12345, timeout=10, test_name='construction'
        )
        self.assertEqual(tc.timeout, 10.0)
        tc.timeout = 0.1
        self.assertEqual(tc.timeout, 0.1)
        tc.timeout = 100
        self.assertEqual(tc.timeout, 100.0)
        tc.timeout = 100
        self.assertEqual(tc.timeout, 100.0)
        del tc.timeout
        self.assertEqual(
            tc.timeout, transmission_rpc.constants.DEFAULT_TIMEOUT
        )
        tc.timeout = '100.1'
        self.assertEqual(tc.timeout, 100.1)
        self.assertRaises(ValueError, tc._set_timeout, '10 years')

    def testAddTorrent(self):
        data_path = os.path.join(
            os.path.dirname(os.path.abspath(__file__)), 'data'
        )

        tc = createClient(test_name='add_torrent_base64')
        torrent_path = os.path.join(
            data_path, 'ubuntu-12.04.2-alternate-amd64.iso.torrent'
        )
        with open(torrent_path, 'rb') as f:
            data = f.read()
        data_b64 = base64.b64encode(data).decode('utf-8')
        r = tc.add_torrent(data_b64)
        self.assertEqual(r.id, 0)
        self.assertEqual(
            r.hashString, 'a21c45469c565f3fb9595e4e9707e6e9d45abca6'
        )
        self.assertEqual(r.name, 'ubuntu-12.04.2-alternate-amd64.iso')

        tc = createClient(test_name='adduri')
        self.assertRaises(ValueError, tc.add_torrent, None)

        r = tc.add_torrent(
            'torrent.txt',
            paused=False,
            download_dir='/var/downloads',
            peer_limit=1
        )
        self.assertEqual(r.id, 0)
        self.assertEqual(r.hashString, 'A000')
        self.assertEqual(r.name, 'testtransfer0')

        r = tc.add_torrent(
            'file://' + os.path.join(data_path, 'torrent.txt'),
            paused=True,
            download_dir='/tmp',
            peer_limit=200
        )
        self.assertEqual(r.id, 1)
        self.assertEqual(r.hashString, 'A001')
        self.assertEqual(r.name, 'testtransfer1')

    def testRemoveTorrent(self):
        tc = createClient(test_name='remove')

        tc.remove_torrent(['b000', 2, 3])
        tc.remove_torrent(1, delete_data=True)
        tc.remove_torrent('b002', delete_data=False)

    def testStartTorrent(self):
        tc = createClient(test_name='start')

        tc.start_torrent(['abcdef', 20, 30])
        tc.start_torrent(1)
        tc.start_torrent('a0123456789')

    def testVerifyTorrent(self):
        tc = createClient(test_name='verify')

        tc.verify_torrent(10000)
        tc.verify_torrent('d')
        tc.verify_torrent(['a', 'b', 'c'])

    def testGetTorrent(self):
        tc = createClient(test_name='get_torrent')
        torrent = tc.get_torrent(2)
        self.assertEqual(torrent.id, 2)
        self.assertEqual(torrent.name, 'ubuntu-10.04-server-amd64.iso')
        self.assertEqual(
            torrent.hashString, 'ab8ea951c022d4745a9b06ab8020b952a52b71ca'
        )

        tc = createClient(test_name='get_torrent_hash')
        torrent = tc.get_torrent('ab8ea951c022d4745a9b06ab8020b952a52b71ca')
        self.assertEqual(torrent.id, 2)
        self.assertEqual(torrent.name, 'ubuntu-10.04-server-amd64.iso')
        self.assertEqual(
            torrent.hashString, 'ab8ea951c022d4745a9b06ab8020b952a52b71ca'
        )

    def testGetTorrents(self):
        tc = createClient(test_name='info')
        r = tc.get_torrents()
        for torrent in r:
            if torrent.id == 2:
                self.assertEqual(torrent.name, 'ubuntu-10.04-server-amd64.iso')
                self.assertEqual(
                    torrent.hashString,
                    'ab8ea951c022d4745a9b06ab8020b952a52b71ca'
                )
            elif torrent.id == 3:
                self.assertEqual(
                    torrent.name, 'ubuntu-10.04-alternate-i386.iso'
                )
                self.assertEqual(
                    torrent.hashString,
                    'a33e98826003515e46ef5075fcbf4914b307abe2'
                )
            else:
                self.fail("Unknown torrent")

    def testGetTorrentsRange(self):
        tc = createClient(test_name='get_torrents_2to3')
        r = tc.get_torrents([2, 3])
        for torrent in r:
            if torrent.id == 2:
                self.assertEqual(torrent.name, 'ubuntu-10.04-server-amd64.iso')
                self.assertEqual(
                    torrent.hashString,
                    'ab8ea951c022d4745a9b06ab8020b952a52b71ca'
                )
            elif torrent.id == 3:
                self.assertEqual(
                    torrent.name, 'ubuntu-10.04-alternate-i386.iso'
                )
                self.assertEqual(
                    torrent.hashString,
                    'a33e98826003515e46ef5075fcbf4914b307abe2'
                )
            else:
                self.fail("Unknown torrent")

        tc = createClient(test_name='get_torrents_2to3')
        r = tc.get_torrents("2:3")
        for torrent in r:
            if torrent.id == 2:
                self.assertEqual(torrent.name, 'ubuntu-10.04-server-amd64.iso')
                self.assertEqual(
                    torrent.hashString,
                    'ab8ea951c022d4745a9b06ab8020b952a52b71ca'
                )
            elif torrent.id == 3:
                self.assertEqual(
                    torrent.name, 'ubuntu-10.04-alternate-i386.iso'
                )
                self.assertEqual(
                    torrent.hashString,
                    'a33e98826003515e46ef5075fcbf4914b307abe2'
                )
            else:
                self.fail("Unknown torrent")

        tc = createClient(test_name='get_torrents_2to3')
        r = tc.get_torrents("2,3")
        for torrent in r:
            if torrent.id == 2:
                self.assertEqual(torrent.name, 'ubuntu-10.04-server-amd64.iso')
                self.assertEqual(
                    torrent.hashString,
                    'ab8ea951c022d4745a9b06ab8020b952a52b71ca'
                )
            elif torrent.id == 3:
                self.assertEqual(
                    torrent.name, 'ubuntu-10.04-alternate-i386.iso'
                )
                self.assertEqual(
                    torrent.hashString,
                    'a33e98826003515e46ef5075fcbf4914b307abe2'
                )
            else:
                self.fail("Unknown torrent")

        tc = createClient(test_name='get_torrents_2to3')
        r = tc.get_torrents("2 3")
        for torrent in r:
            if torrent.id == 2:
                self.assertEqual(torrent.name, 'ubuntu-10.04-server-amd64.iso')
                self.assertEqual(
                    torrent.hashString,
                    'ab8ea951c022d4745a9b06ab8020b952a52b71ca'
                )
            elif torrent.id == 3:
                self.assertEqual(
                    torrent.name, 'ubuntu-10.04-alternate-i386.iso'
                )
                self.assertEqual(
                    torrent.hashString,
                    'a33e98826003515e46ef5075fcbf4914b307abe2'
                )
            else:
                self.fail("Unknown torrent")

    def testGetTorrentsHashes(self):
        tc = createClient(test_name='get_torrents_hashes')
        r = tc.get_torrents([
            "ab8ea951c022d4745a9b06ab8020b952a52b71ca",
            "a33e98826003515e46ef5075fcbf4914b307abe2"
        ])
        for torrent in r:
            if torrent.id == 2:
                self.assertEqual(torrent.name, 'ubuntu-10.04-server-amd64.iso')
                self.assertEqual(
                    torrent.hashString,
                    'ab8ea951c022d4745a9b06ab8020b952a52b71ca'
                )
            elif torrent.id == 3:
                self.assertEqual(
                    torrent.name, 'ubuntu-10.04-alternate-i386.iso'
                )
                self.assertEqual(
                    torrent.hashString,
                    'a33e98826003515e46ef5075fcbf4914b307abe2'
                )
            else:
                self.fail("Unknown torrent")

        tc = createClient(test_name='get_torrents_hashes')
        r = tc.get_torrents(
            "ab8ea951c022d4745a9b06ab8020b952a52b71ca,a33e98826003515e46ef5075fcbf4914b307abe2"
        )
        for torrent in r:
            if torrent.id == 2:
                self.assertEqual(torrent.name, 'ubuntu-10.04-server-amd64.iso')
                self.assertEqual(
                    torrent.hashString,
                    'ab8ea951c022d4745a9b06ab8020b952a52b71ca'
                )
            elif torrent.id == 3:
                self.assertEqual(
                    torrent.name, 'ubuntu-10.04-alternate-i386.iso'
                )
                self.assertEqual(
                    torrent.hashString,
                    'a33e98826003515e46ef5075fcbf4914b307abe2'
                )
            else:
                self.fail("Unknown torrent")

        tc = createClient(test_name='get_torrents_hashes')
        r = tc.get_torrents(
            "ab8ea951c022d4745a9b06ab8020b952a52b71ca a33e98826003515e46ef5075fcbf4914b307abe2"
        )
        for torrent in r:
            if torrent.id == 2:
                self.assertEqual(torrent.name, 'ubuntu-10.04-server-amd64.iso')
                self.assertEqual(
                    torrent.hashString,
                    'ab8ea951c022d4745a9b06ab8020b952a52b71ca'
                )
            elif torrent.id == 3:
                self.assertEqual(
                    torrent.name, 'ubuntu-10.04-alternate-i386.iso'
                )
                self.assertEqual(
                    torrent.hashString,
                    'a33e98826003515e46ef5075fcbf4914b307abe2'
                )
            else:
                self.fail("Unknown torrent")

    def testParseId(self):
        from transmission_rpc.client import parse_torrent_id

        self.assertEqual(parse_torrent_id(None), None)
        self.assertEqual(parse_torrent_id(10), 10)
        self.assertEqual(parse_torrent_id(10.0), 10)
        self.assertEqual(parse_torrent_id(10.5), None)
        self.assertEqual(parse_torrent_id("10"), 10)
        self.assertEqual(parse_torrent_id("A"), "A")
        self.assertEqual(
            parse_torrent_id("a21c45469c565f3fb9595e4e9707e6e9d45abca6"),
            "a21c45469c565f3fb9595e4e9707e6e9d45abca6"
        )
        self.assertEqual(parse_torrent_id("T"), None)
        self.assertEqual(parse_torrent_id([10]), None)
        self.assertEqual(parse_torrent_id((10, 11)), None)
        self.assertEqual(parse_torrent_id({10: 10}), None)

    def testParseIds(self):
        from transmission_rpc.client import parse_torrent_ids

        self.assertEqual(parse_torrent_ids(None), [])
        self.assertEqual(parse_torrent_ids(10), [10])
        self.assertEqual(parse_torrent_ids(10.0), [10])
        self.assertEqual(parse_torrent_ids("10"), [10])
        self.assertEqual(parse_torrent_ids("A"), ["A"])
        self.assertEqual(
            parse_torrent_ids("a21c45469c565f3fb9595e4e9707e6e9d45abca6"),
            ["a21c45469c565f3fb9595e4e9707e6e9d45abca6"]
        )
        self.assertEqual(parse_torrent_ids(",, "), [])
        self.assertEqual(parse_torrent_ids("1,2,3"), [1, 2, 3])
        self.assertEqual(parse_torrent_ids("1:3"), [1, 2, 3])
        self.assertRaises(ValueError, parse_torrent_ids, "A:3")
        self.assertRaises(ValueError, parse_torrent_ids, "T")
        self.assertEqual(parse_torrent_ids([10]), [10])
        self.assertEqual(parse_torrent_ids((10, 11)), [10, 11])
        self.assertRaises(ValueError, parse_torrent_ids, {10: 10})


def suite():
    suite = unittest.TestLoader().loadTestsFromTestCase(ClientTest)
    return suite



from transmission_rpc.client import Client
import pytest
import os
TEST_HOST = os.getenv('TRANSMISSION_HOST', '127.0.0.1')
USERNAME = 'transmission-rpc-username'
PASSWORD = 'example-rpc-password'


@pytest.mark.parametrize(
    'kwargs', [
        Client(TEST_HOST, port=9091, user=USERNAME, password=PASSWORD),
    ]
)
def test_client_parse_url(kwargs):
    client = kwargs
    assert client.url == 'http://{}:9091/transmission/rpc'.format(TEST_HOST)



# -*- coding: utf-8 -*-
# 2008-12, Erik Svensson <erik.public@gmail.com>
# Licensed under the MIT license.

import time
import datetime
import unittest
import transmission_rpc
import transmission_rpc.constants
import transmission_rpc.utils


class torrent(unittest.TestCase):
    def assertPropertyException(self, exception, object, property):
        try:
            getattr(object, property)
        except exception:
            pass
        else:
            self.fail()

    def testConstruction(self):
        self.failUnlessRaises(ValueError, transmission_rpc.Torrent, None, {})
        torrent = transmission_rpc.Torrent(None, {'id': 42})

    def testAttributes(self):
        torrent = transmission_rpc.Torrent(None, {'id': 42})
        self.assertTrue(hasattr(torrent, 'id'))
        self.assertEqual(torrent.id, 42)
        self.assertPropertyException(KeyError, torrent, 'status')
        self.assertPropertyException(KeyError, torrent, 'progress')
        self.assertPropertyException(KeyError, torrent, 'ratio')
        self.assertPropertyException(KeyError, torrent, 'eta')
        self.assertPropertyException(KeyError, torrent, 'date_active')
        self.assertPropertyException(KeyError, torrent, 'date_added')
        self.assertPropertyException(KeyError, torrent, 'date_started')
        self.assertPropertyException(KeyError, torrent, 'date_done')

        self.assertRaises(KeyError, torrent.format_eta)
        self.assertEqual(torrent.files(), {})

        data = {
            'id': 1,
            'status': 4,
            'sizeWhenDone': 1000,
            'leftUntilDone': 500,
            'uploadedEver': 1000,
            'downloadedEver': 2000,
            'uploadRatio': 0.5,
            'eta': 3600,
            'activityDate': time.mktime((2008, 12, 11, 11, 15, 30, 0, 0, -1)),
            'addedDate': time.mktime((2008, 12, 11, 8, 5, 10, 0, 0, -1)),
            'startDate': time.mktime((2008, 12, 11, 9, 10, 5, 0, 0, -1)),
            'doneDate': time.mktime((2008, 12, 11, 10, 0, 15, 0, 0, -1)),
        }

        torrent = transmission_rpc.Torrent(None, data)
        self.assertEqual(torrent.id, 1)
        self.assertEqual(torrent.leftUntilDone, 500)
        self.assertEqual(torrent.status, 'downloading')
        self.assertEqual(torrent.progress, 50.0)
        self.assertEqual(torrent.ratio, 0.5)
        self.assertEqual(torrent.eta, datetime.timedelta(seconds=3600))
        self.assertEqual(torrent.date_active, datetime.datetime(2008, 12, 11, 11, 15, 30))
        self.assertEqual(torrent.date_added, datetime.datetime(2008, 12, 11, 8, 5, 10))
        self.assertEqual(torrent.date_started, datetime.datetime(2008, 12, 11, 9, 10, 5))
        self.assertEqual(torrent.date_done, datetime.datetime(2008, 12, 11, 10, 0, 15))

        self.assertEqual(torrent.format_eta(), transmission_rpc.utils.format_timedelta(torrent.eta))

        torrent = transmission_rpc.Torrent(None, {'id': 42, 'eta': -1})
        self.assertPropertyException(ValueError, torrent, 'eta')

        data = {
            'id': 1,
            'status': 4,
            'sizeWhenDone': 1000,
            'leftUntilDone': 500,
            'uploadedEver': 1000,
            'downloadedEver': 2000,
            'uploadRatio': 0.5,
            'eta': 3600,
            'activityDate': time.mktime((2008, 12, 11, 11, 15, 30, 0, 0, -1)),
            'addedDate': time.mktime((2008, 12, 11, 8, 5, 10, 0, 0, -1)),
            'startDate': time.mktime((2008, 12, 11, 9, 10, 5, 0, 0, -1)),
            'doneDate': 0,
        }

        torrent = transmission_rpc.Torrent(None, data)
        self.assertEqual(torrent.date_done, None)

    def testUnicode(self):
        torrent = transmission_rpc.Torrent(None, {'id': 42, 'name': 'あみ'})
        self.assertEqual(torrent.id, 42)
        repr(torrent)
        str(torrent)


def suite():
    suite = unittest.TestLoader().loadTestsFromTestCase(torrent)
    return suite



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2018 Trim21 <trim21me@hotmail.com>
# Licensed under the MIT license.
import os
import os.path
import re
from setuptools import setup


def _get_version():
    PATH_TO_INIT_PY = \
        os.path.join(
            os.path.abspath(os.path.dirname(__file__)),
            'transmission_rpc',
            '__init__.py'
        )

    with open(PATH_TO_INIT_PY, 'r', encoding='utf-8') as fp:
        try:
            for line in fp.readlines():
                if line:
                    line = line.strip()
                    _version = re.findall(r"^__version__ = '([^']+)'$", line,
                                          re.M)
                    if _version:
                        return _version[0]
        except IndexError:
            raise RuntimeError('Unable to determine version.')


os.environ['PBR_VERSION'] = _get_version()

setup(
    pbr=True,
)



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2014 Erik Svensson <erik.public@gmail.com>
# Licensed under the MIT license.

from transmission_rpc.client import Client
from transmission_rpc.constants import DEFAULT_PORT, DEFAULT_TIMEOUT, PRIORITY, \
    RATIO_LIMIT, LOGGER
from transmission_rpc.error import TransmissionError
from transmission_rpc.session import Session
from transmission_rpc.torrent import Torrent
__all__ = [
    'Client', 'DEFAULT_PORT', 'DEFAULT_TIMEOUT', 'PRIORITY', 'RATIO_LIMIT',
    'LOGGER', 'TransmissionError', 'Session', 'Torrent'
]

__author__ = 'Trim21 <trim21me@hotmail.com>'
__author_email__ = 'trim21me@hotmail.com'
__copyright__ = 'Copyright (c) 2018-2019 Trim21'
__license__ = 'MIT'
__version__ = '2.0.0'



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2014 Erik Svensson <erik.public@gmail.com>
# Licensed under the MIT license.

from transmission_rpc.utils import Field


class Session(object):
    """
    Session is a class holding the session data for a Transmission daemon.

    Access the session field can be done through attributes.
    The attributes available are the same as the session arguments in the
    Transmission RPC specification, but with underscore instead of hyphen.
    ``download-dir`` -> ``download_dir``.
    """

    def __init__(self, client=None, fields=None):
        self._client = client
        self._fields = {}
        if fields is not None:
            self._update_fields(fields)

    def __getattr__(self, name):
        try:
            return self._fields[name].value
        except KeyError:
            raise AttributeError('No attribute %s' % name)

    def __str__(self):
        text = ''
        for key in sorted(self._fields.keys()):
            text += "% 32s: %s\n" % (key[-32:], self._fields[key].value)
        return text

    def _update_fields(self, other):
        """
        Update the session data from a Transmission JSON-RPC arguments dictionary
        """
        if isinstance(other, dict):
            for key, value in other.items():
                self._fields[key.replace('-', '_')] = Field(value, False)
        elif isinstance(other, Session):
            for key in list(other._fields.keys()):
                self._fields[key] = Field(other._fields[key].value, False)
        else:
            raise ValueError('Cannot update with supplied data')

    def _dirty_fields(self):
        """Enumerate changed fields"""
        outgoing_keys = ['peer_port', 'pex_enabled']
        fields = []
        for key in outgoing_keys:
            if key in self._fields and self._fields[key].dirty:
                fields.append(key)
        return fields

    def _push(self):
        """Push changed fields to the server"""
        dirty = self._dirty_fields()
        args = {}
        for key in dirty:
            args[key] = self._fields[key].value
            self._fields[key] = self._fields[key]._replace(dirty=False)
        if len(args) > 0:
            self._client.set_session(**args)

    def update(self, timeout=None):
        """Update the session information."""
        self._push()
        session = self._client.get_session(timeout=timeout)
        self._update_fields(session)
        session = self._client.session_stats(timeout=timeout)
        self._update_fields(session)

    def from_request(self, data):
        """Update the session information."""
        self._update_fields(data)

    def _get_peer_port(self):
        """
        Get the peer port.
        """
        return self._fields['peer_port'].value

    def _set_peer_port(self, port):
        """
        Set the peer port.
        """
        if isinstance(port, int):
            self._fields['peer_port'] = Field(port, True)
            self._push()
        else:
            raise ValueError("Not a valid limit")

    peer_port = property(_get_peer_port, _set_peer_port, None, "Peer port. This is a mutator.")

    def _get_pex_enabled(self):
        """Is peer exchange enabled?"""
        return self._fields['pex_enabled'].value

    def _set_pex_enabled(self, enabled):
        """Enable/disable peer exchange."""
        if isinstance(enabled, bool):
            self._fields['pex_enabled'] = Field(enabled, True)
            self._push()
        else:
            raise TypeError("Not a valid type")

    pex_enabled = property(_get_pex_enabled, _set_pex_enabled, None, "Enable peer exchange. This is a mutator.")
